// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

#pragma warning disable 1591

namespace Twig
{
    /// <summary>
    /// A Person of the size of a child.
    /// Child is attracted to Balls and afraid of other children.
    /// </summary>
    public class Child : Person
    {
        // Need this one because the script interpreter wants the first arg to a constructor always to be the game
        public Child(TwigGame g, string name, float scale, Vector3 position, Vector3 facing)
            : this(g, name, scale, null)
        {
            facing.Normalize();
            PoseAt(position, facing);
        }

        public Child(TwigGame g, string name, float scale, Vector3 position)
            : this(g, name, scale, position, new Vector3(0, 0, 1)) { }

        public Child(TwigGame g, string name, float scale, Adult parent)
            : base(g, name, scale * 0.4f, scale * 0.1f,
                    scale*0.3f, scale*0.075f, scale*0.2f, scale*0.2f,
                    scale*0.3f/2f, scale*0.09f, scale*0.25f, scale*0.25f, scale*0.3f)
        {
            primaryCareGiver = parent;
            Attention.Focus = parent;
            baseLineSecurity = 0.5f;
            actionThreshold = 0.1f;
            maxSpeed *= 2;
        }

        public override void SetupHold(Mammal holder, Grasp grasp)
        {
            holder.Hold(arms.left.root, arms.left.root, this, grasp);
        }

        public void Limp()
        {
            standUp = sitUp = false;
            foreach (Node n in nodes)
            {
                n.ControlMode = ControlMode.Free;
                n.Acceleration = Physics.gVector;
            }
        }
#if MDX
        public override Material DefaultMaterial
        {
            get
            {
                Material m = new Material();
                m.DiffuseColor = new ColorValue(255, 0, 0);
                m.AmbientColor = new ColorValue(100, 0, 0);
                return m;
            }
        }
#endif

#if kids
        public override Mammal.ObjectAppraisal GenericAppraisal(TwigObject o)
        {
            if (o is Ball || o is MerryGoRound)
                return new ObjectAppraisal(0.4f);
            else if (o is Child)
                return new ObjectAppraisal(-0.7f);
            else
                return base.GenericAppraisal(o);
        }
#endif
    }
}
