// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Twig
{
    /// <summary>
    /// Provides a HUD into which objects can print things using the Printf methods.
    /// </summary>
    public class ConsolePrintf : HUD
    {
        /// <summary>
        /// Creates a HUD for displaying printfs from TwigObjects.
        /// </summary>
        /// <param name="g"></param>
        public ConsolePrintf(TwigGame g) : base(g) { }

        /// <summary>
        /// Draws the accumulated printf messages for the currently selected DebugObject
        /// </summary>
        /// <param name="gameTime"></param>
        public override void Draw(GameTime gameTime)
        {
            TwigObject debugObject = TwigGame.DebugObject;
            if (debugObject != null && debugObject.DebugOutput.Count > 0)
            {
                HUDEffect.Begin();
                foreach (EffectPass pass in HUDEffect.CurrentTechnique.Passes)
                {
                    pass.Begin();
                    HUDSprites.Begin();
                    for (int i = 0; i < debugObject.DebugOutput.Count; i++)
                        HUDSprites.DrawString(HUDFont, debugObject.DebugOutput[i],
                            new Vector2(0, i * HUDFont.LineSpacing), Color.White);
                    HUDSprites.End();
                    pass.End();
                }
                HUDEffect.End();
            }
        }

    }
}
