// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

#pragma warning disable 1591

namespace Twig
{
    /// <summary>
    /// A passive doll that can be carried by a Child.
    /// </summary>
    public class Doll : Ragdoll
    {
        public Doll(TwigGame g, string name)
            : base(g, name, 0.1f, 0.025f,
                    0.1f, 0.02f, 0.05f, 0.05f,
                    0.3f / 8f, 0.025f, 0.05f, 0.05f, 0.035f)
        {
            foreach (Link l in Links)
            {
                l.node1.Acceleration = l.node2.Acceleration = 1f*Physics.gVector;
                l.painSensitivity = 0;
            }
        }

#if MDX
        public override Microsoft.DirectX.Direct3D.Material DefaultMaterial
        {
            get
            {
                Material m = new Material();
                m.DiffuseColor = new ColorValue(200, 150, 150);
                m.AmbientColor = m.DiffuseColor;
                return m;
            }
        }
#endif

        // This is so small the inter-link separation due
        // to gravity is noticable, so we run the constraint
        // solver twice to make sure it really converges
        public override void Constrain()
        {
            base.Constrain();
            base.Constrain();
            //base.Constrain();
        }
    }
}
