// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

#pragma warning disable 1591

namespace Twig
{
    partial class Mammal
    {
        /// <summary>
        /// Efferent.  Causes character to punch at target.
        /// </summary>
        protected bool punch;
        /// <summary>
        /// Efferent.  Causes character to grapple with target.
        /// </summary>
        protected bool grapple;

        /// <summary>
        /// Controls fighting behavior.
        /// Initialized in Mammal constructor.
        /// </summary>
        protected FightBehavior fightBehavior;

        /// <summary>
        /// Controls fighting behavior.
        /// </summary>
        protected class FightBehavior : Behavior<ApproachParameters>
        {
            public override string Name
            {
                get
                {
                    return "Fight";
                }
            }
            TwigObject target;
            /// <summary>
            /// Distance from which to punch or grapple.
            /// </summary>
            public float standoffDistance = 0.6f;
            Mammal me;

            public FightBehavior(Mammal me)
            {
                this.me = me;
                standoffDistance = 0.6f * me.arms.left.length;
            }

            protected override void Update()
            {
                // Determine whether to fire
                if (me.Appraisal(me.Attention.Focus).valence <= -me.actionThreshold
                    // && me.DistanceTo(me.Attention.Focus) < 5f
                    // && me.Attention.Focus.LookingAt(me)
                    )
                            target = me.Attention.Focus;

                // Update activation and motor vector
                Activation = (target == null || me.DistanceTo(target) > 5) ? 0 : 0.5f*(me.Appraisal(target).Salience-me.pain/10)/me.DistanceTo(me.Attention.Focus);
                //me.Chart("Fight activation", Activation, 2);

                if (target != null)
                {
                    MotorOutput.Approach(target, standoffDistance);
                    MotorOutput.grapple = me.DistanceTo(target) <= standoffDistance*1.5f;
                }
            }
        }
    }
}
