// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Twig
{
    /// <summary>
    /// Describes a preference for a grasp of an object
    /// </summary>
    [Flags]
    public enum Grasp : byte
    {
        /// <summary>
        /// Use left hand
        /// </summary>
        LeftHanded=1,
        /// <summary>
        /// Use right hand
        /// </summary>
        RightHanded=2,
        /// <summary>
        /// Use dominant hand, whichever it may be
        /// </summary>
        DominantHand=4,
        /// <summary>
        /// Use non-dominant hand, whichever it may be
        /// </summary>
        NonDominantHand=8,
        /// <summary>
        /// Use both hands
        /// </summary>
        TwoHanded=16,
        /// <summary>
        /// Use any hand
        /// </summary>
        AnyHand = LeftHanded | RightHanded,
        /// <summary>
        /// Use any grasp
        /// </summary>
        Any = 255,
        /// <summary>
        /// Can't grasp
        /// </summary>
        None = 0
    }

    /// <summary>
    /// Represents the lack of a free hand for a grasp operation.
    /// </summary>
    public class NoFreeHandException : Exception
    { }
}
