// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Twig
{
    /// <summary>
    /// Displays information on key bindings
    /// </summary>
    public class HelpDisplay : HUD
    {
        /// <summary>
        /// Creates a help display HUD that is toggled by the specified key.
        /// </summary>
        public HelpDisplay(TwigGame g, Keys key) : base(g, key, "Toggle display of keyboard command documentation", false) { }

        /// <summary>
        /// Displays keyboard documentation, if enabled.
        /// </summary>
        public override void Draw(Microsoft.Xna.Framework.GameTime gameTime)
        {
            int lineHeight = TwigGame.HUDFont.LineSpacing;

            TwigGame.HUDSprites.Begin();
            for (int i = 0; i < TwigGame.KeyBindings.Count; i++)
            {
                TwigGame.HUDSprites.DrawString(TwigGame.HUDFont, FormatKey(TwigGame.KeyBindings[i].modifiers, TwigGame.KeyBindings[i].key),
                    new Vector2(0, lineHeight * i), Color.White);
                TwigGame.HUDSprites.DrawString(TwigGame.HUDFont, TwigGame.KeyBindings[i].documentation,
                    new Vector2(50, lineHeight * i), Color.White);
            }
            TwigGame.HUDSprites.End();
        }

        string FormatKey(KeyModifiers m, Keys k)
        {
            if (m == KeyModifiers.None)
                return k.ToString();
            return String.Format("{0}{1}{2}{3}",
                ((m & KeyModifiers.Control) != 0) ? "C-" : "",
                ((m & KeyModifiers.Shift) != 0) ? "S-" : "",
                ((m & KeyModifiers.Alt) != 0) ? "A-" : "",
                k);
        }
    }
}
