// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace Twig
{
    partial class Mammal
    {
        /// <summary>
        /// Efferent.  Triggers hugging of whatever object is immediately in front of creature.
        /// </summary>
        public bool hug = false;
        /// <summary>
        /// Afferent.  True if engaged in hugging.
        /// </summary>
        public bool hugging;
        /// <summary>
        /// True if character is being hugged by another character
        /// </summary>
        public bool beingHugged;

        /// <summary>
        /// Test whether creature is holding object
        /// </summary>
        /// <param name="o">object to test</param>
        /// <returns>True if creature holding object</returns>
        public bool IsHolding(TwigObject o)
        {
            return arms.left.IsGripping(o) || arms.right.IsGripping(o);
        }

        /// <summary>
        /// Controls arms to attempt to grapple with an object, when hug is true.
        /// </summary>
        void HugControl()
        {
            beingHugged = contactObject != null && contactObject is Mammal
                && ((Mammal)contactObject).hugging && ((Mammal)contactObject).approach.target == this;
            //Chart("beingHugged", beingHugged ? 0.5f : -0.5f, 1f);

            GazeLock = hug && approach.target == gazeTarget
                && Vector3.Dot(bodyForward, gazeDirection) > 0f
                && DistanceTo(approach.target) < 3f;

            if (approach.target!=null && IsHolding(approach.target))
            {
                // Hug an object already being held, e.g. ragdoll
                if (hug)
                {
                    //arms.left.Goto(spineTop.Position, 0.5f, ControlMode.Free);
                    //arms.right.Goto(spineTop.Position, 0.5f, ControlMode.Free);
                    arms.left.Goto(arms.right.root.Position, 0.5f, ControlMode.Free);
                    arms.right.Goto(arms.left.root.Position, 0.5f, ControlMode.Free);
                    hugging = true;
                }
                else
                    hugging = false;
            }
            else
            {
                hugging = (arms.left.GripTarget == arms.right.end);
                if (hug)
                {
                    if ((upperSpine.contactObject == approach.target || head.contactObject == approach.target)
                        && Vector3.Dot(approach.target.Position - Position, bodyForward) > 0)
                    {
                        // There's something to hug
                        arms.left.Grab(arms.right.end, null);
                    }
                    else
                        WaitingFor("Hug: target contact");

                    if (!hugging)
                    {
                        if (approach.target == gazeTarget && Vector3.Dot(bodyForward, gazeDirection) > 0f)
                        {
                            reach = true;
                        }
                        else
                            WaitingFor("Hug: face and gaze at target");
                    }
                    else
                        liftArms = true;
                }
                else if (hugging)
                {
                    arms.left.Ungrab();
                }
            }
        }
    }
}
