// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Text;

namespace Twig
{
    partial class Mammal
    {
        class SayAction : Action
        {
            string speech;
            TwigObject addressee;
            bool interrupt;
            enum State { Stop, WaitTurn, WaitReactionTime, WaitFinishedSpeaking };
            State state;
            Mammal me;
            float reactionTime;
            double wakeUpTime;

            public SayAction(Mammal me, string speech, TwigObject addressee, bool interrupt, float reactionTime)
            {
                this.me = me;
                this.speech = speech;
                this.addressee = addressee;
                this.interrupt = interrupt;
                state = State.Stop;
                this.reactionTime = reactionTime;
            }

            public override void Start()
            {
                state = State.WaitTurn;
            }

            public override void Update()
            {
                switch (state)
                {
                    case State.Stop:
                        break;

                    case State.WaitTurn:
                        Mammal a = addressee as Mammal;
                        if (interrupt || (me.DialogBalloonManager.NobodySpeaking && !me.IsManipulating))
                        {
                            state = State.WaitReactionTime;
                            wakeUpTime = Physics.time + reactionTime;
                        }
                        break;

                    case State.WaitReactionTime:
                        if (wakeUpTime < Physics.time)
                        {
                            me.Say(speech);
                            if (addressee!=null)
                                me.LookAt(addressee);
                            state = State.WaitFinishedSpeaking;
                        }
                        break;

                    case State.WaitFinishedSpeaking:
                        if (!me.IsSpeaking)
                            state = State.Stop;
                        break;

                    default:
                        System.Diagnostics.Debug.Fail("Illegal state in SayAction");
                        break;
                }
            }

            public override ActionStatus Status
            {
                get { return state==State.Stop?ActionStatus.Succeed:ActionStatus.Running; }
            }

            public override void Abort()
            {
                state = State.Stop;
            }
        }
    }
}
