// Copyright 2007, 2008, 2009 Ian Horswill
// This file is part of Twig.
//
// Twig is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 of
//  the License, or (at your option) any later version.
//
// Twig is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Twig.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Twig
{
    /// <summary>
    /// Very simple camera controller.  Doesn't understand visibility constraints.
    /// </summary>
    public class StupidCamera : GameComponent
    {
        TwigGame twigGame;
        /// <summary>
        /// Type of shot to use
        /// </summary>
        public CameraHeight height = CameraHeight.ChestLevel;
        /// <summary>
        /// Type of shot to use
        /// </summary>
        public CameraShot shot;
        /// <summary>
        /// Direction from which camera should shoot the subject
        /// </summary>
        public Vector3 CameraAngle = Vector3.UnitZ;
        /// <summary>
        /// Distance to subject normalized to subject's height.
        /// </summary>
        public float subjectDistance = 3.0f;

        float[] cameraHeights = new float[] { 0.15f, 0.7f, 1f, 3f };

        TwigObject subject;

        /// <summary>
        /// A very lame camera controller
        /// </summary>
        /// <param name="g"></param>
        public StupidCamera(TwigGame g)
            : base(g)
        {
            twigGame = g;
            g.DocumentKey(Keys.Left, "Circle camera to the left around subject");
            g.DocumentKey(Keys.Right, "Circle camera to the right around subject");
            g.DocumentKey(Keys.Up, "Truck camera forward");
            g.DocumentKey(Keys.Down, "Truck camera backward");
            g.BindKey(KeyModifiers.Control, Keys.Up, "Switche between chest level, eye level, bird's eye, and worm's eye",
                delegate(Keys ignore)
                {
                    height++;
                    if (height > CameraHeight.BirdsEye)
                        height = CameraHeight.BirdsEye;
                }
            );
            g.BindKey(KeyModifiers.Control, Keys.Down, "Switche between chest level, eye level, bird's eye, and worm's eye",
               delegate(Keys ignore)
               {
                    height--;
                    if ((int)height < 0)
                        height = (CameraHeight)0;
               }
            );

        }

        /// <summary>
        /// Move the game's viewpoint as appropriate
        /// </summary>
        public override void Update(GameTime gameTime)
        {
            if (twigGame.KeyModifiers == KeyModifiers.None)
            {
                if (twigGame.IsKeyDown(Keys.Left))
                    CameraAngle = Vector3.Transform(CameraAngle, Matrix.CreateRotationY(-(float)Physics.deltaT));
                else if (twigGame.IsKeyDown(Keys.Right))
                    CameraAngle = Vector3.Transform(CameraAngle, Matrix.CreateRotationY((float)Physics.deltaT));
                CameraAngle.Normalize();

                if (twigGame.IsKeyDown(Keys.Up))
                    subjectDistance -= 0.05f;
                else if (twigGame.IsKeyDown(Keys.Down))
                    subjectDistance += 0.05f;
            }

            if (twigGame.DebugObject != null)
                subject = twigGame.DebugObject;

            if (subject == null)
                return;
            float cameraDistance;
            float cameraHeight;

            Mammal m = subject as Mammal;
            if (m != null)
            {
                cameraDistance = m.Height * subjectDistance;
                cameraHeight = m.EyeLevel*cameraHeights[(int)height];
                twigGame.CameraTargetLocation = 0.9f*twigGame.CameraTargetLocation+ 0.1f*(m.Position + ((height==CameraHeight.ChestLevel)?cameraHeight:m.EyeLevel) * Vector3.Up);
            }
            else
            {
                cameraDistance = 10f;
                cameraHeight = 2f;
                twigGame.CameraTargetLocation = subject.Position;
            }

            twigGame.CameraPosition =
                0.9f * twigGame.CameraPosition
                + 0.1f * (subject.Position+CameraAngle*cameraDistance+Vector3.Up*cameraHeight);
            base.Update(gameTime);
        }
    }
}
